/*
	Geochron Blue
	1/15/09, Copyright Sparkfun Electronics
	Pete Dokter
*/
	


/* *********************************************************
               Function declarations
  ********************************************************* */

void Initialize(void);
void feed(void);

void setup_uart0(int baud, char want_ints);
void setup_uart1(int baud, char want_ints);

char get_char(void);

void stat0_on (void);//green
void stat0_off (void);

void stat1_on (void);//blue
void stat1_off (void);

void stat2_on (void);//red
void stat2_off (void);

void config_menu(void);
void change_settings(void);
void put_char(char c);
char getc2(void);

void BT_on(void);
void BT_off(void);

void GPS_on(void);
void GPS_off(void);

void test_code(void);
void mode_0(void);
void Log_init(void);
void get_GPS(char time);
void set_time(void);
void mode_1(void);
void store_GPS(void);

void qprintf(const char *nate, int my_byte);
short bin2Hex(short x);


static void __attribute__ ((interrupt("IRQ"))) UART0ISR(void);

/**********************************************************
                  Header files
 **********************************************************/
#include "LPC21xx_SFE.h"
#include "rprintf.h"
#include "fat16.h"
#include "sd_raw.h"
#include "rootdir.h"
#include <string.h>
#include <stdio.h>
//

/**********************************************************
                  Global Variables
 **********************************************************/
 
char port = 1;        //determines which port will be written to

char temp_array[5];
char asc;

char check_BT = 1;

struct fat16_file_struct * fd;
struct fat16_fs_struct* fs;
struct fat16_dir_entry_struct* dir_entry;

char RX_array1[500];
char RX_array2[500];
char log_array1 = 0;
char log_array2 = 0;
static char GPS_short[500];     //
char interval = 10;   //this is the alarm interval in minutes, default = 10
volatile short RX_in = 0;
short RX_printed = 0;
float battery;
float temperature;
char get_frame = 0;
char set_WAAS = 0;

signed int stringSize = 500;
signed char handle;
char stringBuf[500];
char title[10];

//Default settings=================================================================================
static char mode = 0;    //mode, 0 = auto, 1 = delayed
static char log_what = 0x38;    //RMC, GSA, GGA
static char hour_delay = 0;
static char min_delay = 10;   //10 minute delay default in mode 1
static char sec_delay = 0;
static int holdoff = 5;
static char WAAS = 0;
static char BT_active = 1;
/**********************************************************
                       MAIN
**********************************************************/


int	main (void)
{	
	int	k, j;
	
	Initialize();
	
	//rprintf_devopen(put_char); /* init rrprintf */
	
	port = 1;
    setup_uart1(115200,0);
	
	//POwer up LED sequence=====================================
	for (j = 0; j < 5; j++)
	{
		stat0_on();
		for (k = 0; k < 75000; k++);
		stat0_off();
		stat1_on();
		for (k = 0; k < 75000; k++);
		stat1_off();
		stat2_on();
		for (k = 0; k < 75000; k++);
		stat2_off();
	}
	
	if (sd_raw_init() == 0)
	{
		while(1)
		{
			stat2_on();
			for (j = 0; j < 800000; j++);
			stat2_off();
			for (j = 0; j < 800000; j++);
		}
	}
	
	openroot();
    
	Log_init();
	    
	test_code();
	
	//Indicate good test with another LED blinky======================================
	for (j = 0; j < 5; j++)
	{
		stat0_on();
		for (k = 0; k < 75000; k++);
		stat0_off();
		stat1_on();
		for (k = 0; k < 75000; k++);
		stat1_off();
		stat2_on();
		for (k = 0; k < 75000; k++);
		stat2_off();
	}
	
	if (BT_active == 0) BT_off();
	
	/*
	if (BT_active == 1)
	{
		BT_on();
		
		for (k = 0; k < 1000000; k++);
		for (k = 0; k < 1000000; k++);
		for (k = 0; k < 1000000; k++);
		for (k = 0; k < 1000000; k++);
		
		stat0_on();
		rprintf("$$$",0);
		while(getc2() != 'D');
		stat0_off();
		
		stat1_on();
		rprintf("SI,0012\r",0);
		while(getc2() != 'K');
		stat1_off();
		
		rprintf("SN,Geochron\r",0);
		while(getc2() != 'K');
		
		rprintf("---",0);
	}
	
	else BT_off();
	*/
	
	for (j = 0; j < 255; j++)
    {
        sprintf(title,"LOG%d.txt",j);
		if (root_file_exists((char*) title) == 0) break;
        
    }
	
	fd = root_open_new((char*) title);
	
    if(fd == NULL)//Error creating new log file, can't create new file
	{
		
		while(1)
		{
			stat2_on();
			for (k = 0; k < 750000; k++);
			stat2_off();
			for (k = 0; k < 750000; k++);
		}
			
		
	}
	
	
    if (mode == 0) mode_0();
	
	else
	{
		setup_uart0(4800, 0);
		GPS_on();

		get_GPS(1);
		store_GPS();
		//mode 1 and 2 will go here
		mode_1();
	}
	
	while(1)
	{
		config_menu();
		change_settings();
	}
		
}



/**********************************************************
                      Initialize
**********************************************************/

#define PLOCK 0x400

void Initialize(void)
{
    // Setting Multiplier and Divider values
  	PLLCFG=0x23;
  	feed();
  
	// Enabling the PLL */	
	PLLCON=0x1;
	feed();
  
	// Wait for the PLL to lock to set frequency
	while(!(PLLSTAT & PLOCK)) ;
  
	// Connect the PLL as the clock source
	PLLCON=0x3;
	feed();
  
	// Enabling MAM and setting number of clocks used for Flash memory fetch (4 cclks in this case)
	MAMCR=0x2;
	MAMTIM=0x4;
  
	// Setting peripheral Clock (pclk) to System Clock (cclk)
	VPBDIV=0x01;
	
    //PINSEL0 = 0x000500C5;   // enable uart0, enable uart1 with flow control, 
	PINSEL0 = 0x000500C5;   // enable uart0, enable uart1 with flow control,p0.3 as EINT1
	PINSEL0 |= 0x00001500;   // enable MISO, MOSI, SCK for SPI0 (CS will be an output)
	PINSEL1 |= 0x00000300;	//p0.20 as EINT3
							
	PINSEL2 = 0x00000000;       //make sure the trace pins are GPIO
	
	IODIR0 = 0x40028000;    //P0.17 = Stat1, P0.15 = stat2, P0.30 = GPS_EN
	IODIR1 = 0x00110000;	//P1.20 = STAT0, P1.16 = BT_EN
	
	IOSET0 = 0x00028000;	// Stat0 & Stat1 high
	IOSET1 = 0x00100000;	//Stat2 high
	
	IOCLR0 = 0x00010000;	//BT off
	
	EXTPOLAR = 8;//EINT1 is low active, EINT3 is high
	
	VICIntSelect |= 0x00002000;//RTC
    
}


void feed(void)
{
  PLLFEED=0xAA;
  PLLFEED=0x55;
}


//static void UART0ISR(void)
static void __attribute__ ((interrupt("IRQ"))) UART0ISR(void)
{
      char temp = 0;
      
      temp = U0RBR;

      if (RX_in < 500)
      {
          if (temp != 0)
          {
              RX_array1[RX_in] = temp;
              RX_in++;
          }

          if (RX_in == 500) log_array1 = 1;
      
      }

      else if (RX_in >= 500)
      {
          if (temp != 0)
          {
            RX_array2[RX_in-500] = temp;
            RX_in++;
          }


          if (RX_in == 1000)
          {
              log_array2 = 1;
              RX_in = 0;
          }
      }
      
      temp = U0IIR;	//have to read this to clear the int

      // Update VIC priorities 
      VICVectAddr = 0;
      
}


void put_char(char c)
{ 

    if (port == 0)
    {
        while((U0LSR & 0x20) == 0);
        U0THR = c;
    }
    else if (port == 1)
    {
		while(IOPIN0 & 0x00000800);//Check CTS
        while((U1LSR & 0x20) == 0);
        U1THR = c;
    }
}


//specific to UART0
char get_char(void)
{
    while((U0LSR & 0x01) == 0);
    return (U0RBR);
}


char getc2(void)
{
    while((U1LSR & 0x01) == 0);
    return (U1RBR);
}


void stat0_on (void)
{
	IOCLR1 = 0x00100000;

}

void stat0_off (void)
{
	IOSET1 = 0x00100000;
	
}

void stat1_on (void)
{
	IOCLR0 = 0x00020000;

}

void stat1_off (void)
{
	IOSET0 = 0x00020000;
	
}

void stat2_on (void)
{
	IOCLR0 = 0x00008000;

}

void stat2_off (void)
{
	IOSET0 = 0x00008000;
	
}

void BT_on(void)
{
	IOSET1 = 0x00010000;
}

void BT_off(void)
{
	IOCLR1 = 0x00010000;
}


void GPS_on(void)
{
	IOSET0 = 0x40000000;
}

void GPS_off(void)
{
	IOCLR0 = 0x40000000;
}



void config_menu(void)
{
	char a, b = 0, tempbuffer[50];
	int m;
	
	setup_uart0(4800, 0);
	setup_uart1(115200,0);
	
	GPS_off();
	port = 1;
	
	m = sd_raw_init();
	
	stat0_on();
	stat1_on();
	stat2_on();
	
	for (m = 0; m < 1000000; m++);
	
    qprintf("\r\n\r\n",0);

	while (1)
	{
		qprintf("\r\nGeochron File Manager and Diagnostics\r\n",0);
		qprintf("========================================\r\n",0);
		
		qprintf("1) Read directory\r\n",0);
		qprintf("2) Read file\r\n",0);
		qprintf("3) Delete file\r\n",0);
		qprintf("4) Format SD card\r\n",0);
		qprintf("5) Listen to GPS (press ^s to stop)\r\n",0);
		qprintf("6) Create a test log file\r\n",0);
		qprintf("7) Change settings in the config file\r\n",0);
		
		qprintf("9) Exit menu and reset Geochron\r\n",0);
		

		U1FCR = 0x02;   //Clear the RX fifo

		if (port == 1) a = getc2();
        else a = get_char();
		
              
		if (a == '1')
		{
			rootDirectory_files_stream(1);
			qprintf("\r\n\r\n",0);
			b = 1;
			while (b != 0)
			{
				b = rootDirectory_files_stream(0);
				
				if (b == ',')
				{
					put_char(10);
					put_char(13);
				}
				else put_char(b);
			}
			qprintf("\r\n\r\n",0);
			b = 0;
			
		}

		
		else if (a == '2')
		{
			for (m = 0; m < 50; m++) tempbuffer[m] = 0;
			qprintf("\r\nEnter the name of the file you wish to read: ",0);
			for (m = 0; m < 50; m++)
			{
				b = getc2();
				
				if ((b == 10) | (b == 13)) break;
				put_char(b);
				tempbuffer[m] = b;
				
			}
			
			qprintf("\r\n\r\n",0);
			
			fd = root_open((char*)tempbuffer);
			
			if (fd == 0)
			{
				qprintf("\r\nFailed to open file.\r\n",0);
				break;
			}
			
			while(m > 0)
			{
				tempbuffer[0] = 0;
				
				m = fat16_read_file(fd, (uint8_t*)tempbuffer, 1);
				if (m < 0) 
				{
					qprintf("\r\n\r\nFailed to read %d\r\n\r\n",m);
					break;
				}
				
				if (tempbuffer[0] != 0) put_char(tempbuffer[0]);
				
			}
			
			fat16_close_file(fd);
			sd_raw_sync();
			
			qprintf("\r\n\r\nEnd.\r\n\r\n",0);
		}

		else if (a == '3')
		{			
			for (m = 0; m < 50; m++) tempbuffer[m] = 0;
			qprintf("\r\nEnter the name of the file you wish to delete: ",0);
			for (m = 0; m < 50; m++)
			{
				b = getc2();
				
				if ((b == 10) | (b == 13)) break;
				put_char(b);
				tempbuffer[m] = b;
				
			}
			
			b = root_delete((char*)tempbuffer);
			if (b == 0) qprintf("\r\nDone.\r\n",0);
			else qprintf("\r\nFailed.\r\n",0);
			
			sd_raw_sync();
		}
		
		else if (a == '4')
		{
			root_format();
		}
				
		else if (a == '5')
		{	
			stat0_off();
			stat1_off();
			stat2_off();
			
			GPS_on();
			
			while(1)
			{
				if ((U0LSR & 0x01) != 0)	//if something comes in on port0...
				{				
					a = U0RBR;	//gotta read the RX buffer to set the flag back to 0
					
					U1THR = a;
		  
				}
			  
				if ((U1LSR & 0x01) != 0)	//if something comes in on port1...
				{
					a = U1RBR;	//gotta read the RX buffer to set the flag back to 0
					
					if (a == 19)	//^s to stop and go back to the menu
					{
						break;
					}
					   
					while((U0LSR & 0x40) != 0x40);	//bit 6 in U0LSR is TEMT
					U0THR = a;
					  
				}
			}
			
			GPS_off();
			
			stat0_on();	
			stat1_on();
			stat2_on();
		
		}
		
		else if (a == '6')
		{
			stat0_off();
			stat1_off();
			stat2_off();
			
			fd = root_open_new("Testlog.txt");
			if(fd == NULL)
			{
				qprintf("\r\nError creating Testlog.txt, can't create new file.\n\r",0);
				
				stat0_on();	
				stat1_on();
				stat2_on();
				break;
				
			}
			
			qprintf("\r\nYour Geochron will now create a new GPS log file on your SD card\r\n",0);
			qprintf("\r\ncalled Testlog.txt. To stop and return to the menu, press the space bar.\r\n",0);
			
			check_BT = 0;//make sure not to bail out of mode0 because of the BT connection
			
			mode_0();
			
			check_BT = 1;
			
			port = 1;
			
			stat0_on();	
			stat1_on();
			stat2_on();
			
		}
		
        else if (a == '7')
		{
			b = root_delete("GLOGCON.TXT");
			
			if (b == 0) qprintf("\r\n\r\nGLOGCON.txt deleted.\r\n\r\n",0);
			
			else qprintf("\r\n\r\nCould not delete GLOGCON.txt.\r\n\r\n",0);
			
			qprintf("Here comes the settings menu...\r\n\r\r\n",0);
			
			sd_raw_sync();
			return;
		}
		
		
		else if (a == '9')
		{
			stat0_off();
			stat1_off();
			stat2_off();
			
			qprintf("\r\nResetting...\r\n",0);
			
			//set up the watch dog timer
			WDTC = 0;
			WDMOD = 3;
			WDFEED = 0xAA;
			WDFEED = 0x55;
			
			while(1);
		}
                
				
		
	}
        
        
        
}


	
void setup_uart0(int baud, char want_ints)
{
	
	//set up uart0 
	U0LCR = 0x83;			// 8 bits, no Parity, 1 Stop bit, DLAB = 1
	
	
	if (baud == 1200)
	{
		U0DLM   = 0x0C;		  	//
  		U0DLL 	= 0x00;        	// 4800 Baud Rate @ 58982400 VPB Clock
	}
	
	if (baud == 2400)
	{
		U0DLM   = 0x06;		  	//
  		U0DLL 	= 0x00;        	// 4800 Baud Rate @ 58982400 VPB Clock
	}
		
	if (baud == 4800)
	{
		U0DLM   = 0x03;		  	//
  		U0DLL 	= 0x00;        	// 4800 Baud Rate @ 58982400 VPB Clock
	}
	
	else if (baud == 9600)
	{
	 	U0DLM   = 0x01;		  	//
  		U0DLL 	= 0x80;        	// 9600 Baud Rate @ 58982400 VPB Clock
	}
	
	else if (baud == 19200)
	{
	 	U0DLM   = 0x00;		  	//
  		U0DLL 	= 0xC0;        	// 19200 Baud Rate @ 58982400 VPB Clock
	}
	
	else if (baud == 38400)
	{
	 	U0DLM   = 0x00;		  	//
  		U0DLL 	= 0x60;        	// 38400 Baud Rate @ 58982400 VPB Clock
	}

        else if (baud == 57600)
	{
	 	U0DLM   = 0x00;		  	//
  		U0DLL 	= 0x40;        	// 57600 Baud Rate @ 58982400 VPB Clock
	}

        else if (baud == 115200)
	{
	 	U0DLM   = 0x00;		  	//
  		U0DLL 	= 0x20;        	// 115200 Baud Rate @ 58982400 VPB Clock
	}
	                    
  	U0FCR	= 0x01;			// Not sure this is necessary, but the manual sez it is
	U0LCR 	= 0x03;        	// DLAB = 0
	
	if(want_ints == 1)
	{
		VICIntSelect &= ~0x00000040;
		VICIntEnable |= 0x00000040;
		VICVectCntl0 = 0x26;
		VICVectAddr0 = (unsigned int)UART0ISR;
		
		U0IER = 0x01;
		U0FCR &= 0x3F;

	}
	
	else if (want_ints == 0)
	{
		VICIntEnClr = 0x00000040;
		U0IER = 0;
		//VICIntSelect = 0;
		//VICIntEnable = 0;
	}
	
}


void setup_uart1(int baud, char want_ints)
{
	//set up uart1
	U1LCR 	= 0x83;			// 8 bits, no Parity, 1 Stop bit, DLAB = 1
	
	if (baud == 1200)
	{
		U1DLM   = 0x0C;		  	//
  		U1DLL 	= 0x00;        	// 4800 Baud Rate @ 58982400 VPB Clock
	}
	
	if (baud == 2400)
	{
		U1DLM   = 0x06;		  	//
  		U1DLL 	= 0x00;        	// 4800 Baud Rate @ 58982400 VPB Clock
	}
	
	if (baud == 4800)
	{
		U1DLM   = 0x03;		  	//
  		U1DLL 	= 0x00;        	// 4800 Baud Rate @ 58982400 VPB Clock
	}
	
	else if (baud == 9600)
	{
	 	U1DLM   = 0x01;		  	//
  		U1DLL 	= 0x80;        	// 9600 Baud Rate @ 58982400 VPB Clock
	}
	
	else if (baud == 19200)
	{
	 	U1DLM   = 0x00;		  	//
  		U1DLL 	= 0xC0;        	// 19200 Baud Rate @ 58982400 VPB Clock
	}
	
	else if (baud == 38400)
	{
	 	U1DLM   = 0x00;		  	//
  		U1DLL 	= 0x60;        	// 38400 Baud Rate @ 58982400 VPB Clock
	}

        else if (baud == 57600)
	{
	 	U1DLM   = 0x00;		  	//
  		U1DLL 	= 0x40;        	// 57600 Baud Rate @ 58982400 VPB Clock
	}

        else if (baud == 115200)
	{
	 	U1DLM   = 0x00;		  	//
  		U1DLL 	= 0x20;        	// 115200 Baud Rate @ 58982400 VPB Clock
	}
	    
  	U1FCR	= 0x01;			// Not sure this is necessary, but the manual sez it is
  	
	U1LCR 	= 0x03;        	// DLAB = 0
	
	
	if (want_ints == 1)
	{
			/*
            stat1_on();
            // UART1 interrupt is an IRQ interrupt 
            VICIntSelect &= ~0x80;
            // Enable UART interrupt 
            VICIntEnable = 0x80;
            // Use slot 0 for UART1 interrupt 
            VICVectCntl0 = 0x27;
            // Set the address of ISR for slot 0 
            VICVectAddr0 = (unsigned int)UART1ISR;

            //VICIntSelect |= 0x00000080;		//bit 7 is for UART1, sets to FIQ (not IRQ)
            //VICIntEnable |= 0x00000080;	//enables UART1 interupt
            					
			*/
			U1IER = 0x01;//enable RX data available interupt, DLAB must be 0 to write
	}
	
	else if (want_ints == 0)
	{
		VICIntEnClr = 0x00000080;	//disables UART1 interupt
		U1IER = 0x00;					//disable RX data available interupt, DLAB must be 0 to write
	}
	

}





void test_code(void)
{
    int x;
    char temp;
	
	stat0_off();
	stat1_off();
	stat2_off();

    setup_uart0(4800,0);

    CCR = 0;
    CCR = 16;		//Enable RTC config

    HOUR = 0;
    MIN = 0;
    SEC = 0;
    
    CCR = 17;//Run RTC

    temp = SEC;
	stat2_on();//red LED stays on during test
    while(temp == SEC);//test to see that the RTC is counting
	stat2_off();
    

    port = 0;//send command for WAAS
	GPS_on();
	stat0_on();//green LED is stays on during test
    for (x = 0; x < 5000000; x++);
    qprintf("$PSRF151,01*0F",0); //set for WAAS
    put_char(13);
    put_char(10);
    

    while(get_char() != 'N');
    while(get_char() != 'M');
    while(get_char() != 'E');
    while(get_char() != 'A');
    while(get_char() != '1');
    while(get_char() != '5');
    while(get_char() != '1');
	
	GPS_off();//we turn the GPS off to get rid of the WAAS setting
	//if the user has configured it to be on, it will get set elsewhere
	
	stat0_off();
	
	
	BT_on();
	port = 1;
	stat1_on();//blue LED is stays on during test
	
	//wait for Bluetooth to power up
	for (x = 0; x < 1000000; x++);
	for (x = 0; x < 1000000; x++);
	for (x = 0; x < 1000000; x++);
	for (x = 0; x < 1000000; x++);
	
	
	
	qprintf("$$$",0);
	while(getc2() != 'D');
	

	qprintf("SI,0012\r",0);
	while(getc2() != 'K');
	
	
	qprintf("SN,Geochron\r",0);
	while(getc2() != 'K');
	
	qprintf("---",0);
	
	stat1_off();
	return;
	
}

//Auto UART mode =============================================================================================
void mode_0(void)
{
    setup_uart0(4800, 1);
    int j;
	unsigned char a;
	
	GPS_on();
	for (j = 0; j < 1200000; j++);
	

    while(1)
    {
        if (log_array1 == 1)
        {
            stat0_on();
			
			if (fat16_write_file(fd, (unsigned char*)RX_array1, stringSize) < 0)
            {
				port = 1;
				qprintf("\r\nCan't write to file(1).\r\n",0);
                while(1)
                {
                    stat2_on();
                    for (j = 0; j < 500000; j++);
                    stat2_off();
                    for (j = 0; j < 500000; j++);
                }
            }

            a = sd_raw_sync();
			
            stat0_off();
            log_array1 = 0;

            //We wait briefly after power up to enable WAAS
            if (WAAS == 1)
            {
                port = 0;

                stat0_on();
                qprintf("$PSRF151,01*0F",0); //set for WAAS
                put_char(13);
                put_char(10);
                port = 1;

                WAAS = 0;
                for (j = 0; j < 120000; j++);
                stat0_off();
            }
        }

        if (log_array2 == 1)
        {
            stat1_on();
			
            if (fat16_write_file(fd, (unsigned char*)RX_array2, stringSize) < 0) //stop if we can't write
            {
				port = 1;
				qprintf("\r\nCan't write to file(2).\r\n",0);
                while(1)
                {
                    stat2_on();
                    for (j = 0; j < 500000; j++);
                    stat2_off();
                    for (j = 0; j < 500000; j++);
                }
            }
					
            a = sd_raw_sync();
			
            stat1_off();
            log_array2 = 0;
        }
		
		//If we're running normally and get a BT connection, return
		if (check_BT == 1)
		{
			if (IOPIN0 & 0x00100000) return;
			
		}
		
		//If we're doing this from the menu over BT, check for space bar to return
		if (U1LSR & 0x01) a = U1RBR;
		
		if (a == 32) return;
		        
        //if the button has been pushed, log the rest of the file and quit
        if ((IOPIN0 & 0x00010000) == 0)    
        {
            VICIntEnClr = 0xFFFFFFFF;  //clear all interrupts
            GPS_off();

            if (RX_in < 500)
            {
				fat16_write_file(fd, (unsigned char*)RX_array1, RX_in);
				sd_raw_sync();
                
            }
    
            else if (RX_in >= 500)
            {
                fat16_write_file(fd, (unsigned char*)RX_array2, RX_in - 512);
				sd_raw_sync();
                
            }
			
			if (IOPIN0 & 0x00100000) return;//emergency bailout
			
			while(1)
			{
				stat2_on();
				for (j = 0; j < 500000; j++);
				stat2_off();
				for (j = 0; j < 500000; j++);
				
			}
			
			
        }
    }


}


void Log_init(void)
{
    int j, x, mark = 0, indx = 0;
    char temp;
	
	for (j = 0; j < stringSize; j++) stringBuf[j] = 0;
    
	fd = root_open("GLOGCON.TXT");
	
	if (fd != NULL)
	{
		j = fat16_read_file(fd, (uint8_t*)stringBuf, stringSize);
		stringBuf[stringSize] = '\0';

	}
		
    else
    {
        fd = root_open_new("GLOGCON.TXT");
		
		if (fd != NULL)
        {
			x = 0;
			
			//Set up the array for the new config file======================================================================================
			strcpy(RX_array1, "Mode = Log What = Time Between Logs = Holdoff = WAAS = Bluetooth Active = ");
			
			for (j = 0; j < 75; j++)
			{
				if (j == 7)
				{
					stringBuf[x] = mode + 48;
					stringBuf[x+1] = 13;
					stringBuf[x+2] = 10;
					x += 3;
					
				}
				
				else if (j == 18)
				{
					if (log_what & 0x20)//RMC
					{
						stringBuf[x] = 'R';
						stringBuf[x+1] = 'M';
						stringBuf[x+2] = 'C';
						x += 3;
					}
					
					if (log_what & 0x10)//GGA
					{
						stringBuf[x] = ';';
						stringBuf[x+1] = 'G';
						stringBuf[x+2] = 'G';
						stringBuf[x+3] = 'A';
						x += 4;
					}
					
					if (log_what & 0x08)//GSA
					{
						stringBuf[x] = ';';
						stringBuf[x+1] = 'G';
						stringBuf[x+2] = 'S';
						stringBuf[x+3] = 'A';
						x += 4;
					}
					
					if (log_what & 0x04)//GSV
					{
						stringBuf[x] = ';';
						stringBuf[x+1] = 'G';
						stringBuf[x+2] = 'S';
						stringBuf[x+3] = 'V';
						x += 4;
					}
					
					if (log_what & 0x02)//VTG
					{
						stringBuf[x] = ';';
						stringBuf[x+1] = 'V';
						stringBuf[x+2] = 'T';
						stringBuf[x+3] = 'G';
						x += 4;
					}
					
					if (log_what & 0x01)//GLL
					{
						stringBuf[x] = ';';
						stringBuf[x+1] = 'G';
						stringBuf[x+2] = 'L';
						stringBuf[x+3] = 'L';
						x += 4;
					}
					
					stringBuf[x] = 13;
					stringBuf[x+1] = 10;
					x += 2;
				}
				
				else if (j == 38)
				{
					//hours
					stringBuf[x] = hour_delay / 10 + 48;
					stringBuf[x+1] = hour_delay % 10 + 48;
					stringBuf[x+2] = ':';
					
					//minutes
					stringBuf[x+3] = min_delay / 10 + 48;
					stringBuf[x+4] = min_delay % 10 + 48;
					stringBuf[x+5] = ':';
					
					//seconds
					stringBuf[x+6] = sec_delay / 10 + 48;
					stringBuf[x+7] = sec_delay % 10 + 48;

					x += 8;
					
					stringBuf[x] = 13;
					stringBuf[x+1] = 10;
					x += 2;
				}
				
				else if (j == 48)
				{
					
					if (holdoff >= 10)
					{
						stringBuf[x] = holdoff / 10 + 48;
						x++;
					}
					
					stringBuf[x] = holdoff % 10 + 48;
					x++;
					
					stringBuf[x] = 13;
					stringBuf[x+1] = 10;
					x += 2;
				}
				
				else if (j == 55)
				{
					stringBuf[x] = WAAS + 48;
					x++;
					
					stringBuf[x] = 13;
					stringBuf[x+1] = 10;
					x += 2;
				}
				
				else if (j == 74)
				{
					stringBuf[x] = BT_active + 48;
					x++;
					
					stringBuf[x] = 13;
					stringBuf[x+1] = 10;
					x += 2;
					break;
				}
					
					
				
				stringBuf[x] = RX_array1[j];
				x++;

			}

			
			fat16_write_file(fd, (unsigned char*)stringBuf, x);

            sd_raw_sync();
			
			return;				

        }

        else
        {
            while(1)    //in case there's no card or it's full...
            {
                stat2_on();
                for (j = 0; j < 500000; j++);
                stat2_off();
                for (j = 0; j < 500000; j++);
                
            }
        }
    }
    
    
    for (x = 0; x < stringSize; x++)
    {
        temp = stringBuf[x];

        if (temp == 10)
        {
            mark = x;
            indx++;

            if (indx == 1)
            {
                mode = stringBuf[mark - 2] - 48; //0 = auto uart, 1 = delayed
                if ((mode != 0) && (mode != 1) && (mode != 2)) mode = 0;

            }

            else if (indx == 2)
            {
                
                mark-=1;
                log_what = 0;
				
                while(stringBuf[mark] != ' ')
                {
                    
                    if (stringBuf[mark - 1] == 'C') log_what |= 0x20;        //RMC, bit 5
                    else if ((stringBuf[mark - 1] == 'A') && (stringBuf[mark - 2] == 'G'))
                    {
                        log_what |= 0x10;   //GGA, bit 4
            
                    }
                    else if ((stringBuf[mark - 1] == 'A') && (stringBuf[mark - 2] == 'S'))
                    {
                        log_what |= 0x08;   //GSA, bit 3

                    }

                    else if (stringBuf[mark - 1] == 'V') log_what |= 0x04;   //GSV, bit 2
                    else if (stringBuf[mark - 1] == 'G') log_what |= 0x02;   //VTG, bit 1
                    else if (stringBuf[mark - 1] == 'L') log_what |= 0x01;   //GLL, bit 0

                    while(1)
                    {
                        mark--;
                        if ((stringBuf[mark] == ';') || (stringBuf[mark] == ' ')) break;
                    }
                    
                }


            }
        
            else if (indx == 3)
            {
                sec_delay = (stringBuf[mark - 2] - 48) + (stringBuf[mark - 3] - 48)*10;
                min_delay = (stringBuf[mark - 5] - 48) + (stringBuf[mark - 6] - 48)*10;
                hour_delay = (stringBuf[mark - 8] - 48);
                if ((stringBuf[mark - 9] > 48) && (stringBuf[mark - 9] < 58)) hour_delay += (stringBuf[mark - 9] - 48)*10;

                if (hour_delay > 23)    //This should prevent a funky rollover.  Max delay is 24 hours.
                {
                    sec_delay = 0;
                    min_delay = 0;
                    hour_delay = 24;
                }

            }

            else if (indx == 4)
            {
                holdoff = (stringBuf[mark - 2] - 48);
                if ((stringBuf[mark - 3] >= 48) & (stringBuf[mark - 3] <= 57)) holdoff += (stringBuf[mark - 3] - 48)*10;
                
                if ((holdoff > 99) | (holdoff < 1)) holdoff = 5;

            }

            else if (indx == 5)
            {
                if (stringBuf[mark - 2] == '0') WAAS = 0;
                else if (stringBuf[mark - 2] == '1') WAAS = 1;
                
            }
			
			else if (indx == 6)
            {
                if (stringBuf[mark - 2] == '0') BT_active = 0;
				else if (stringBuf[mark - 2] == '1') BT_active = 1;
                
            }

            
        }

    }
	
}


//This is the parser for mode_1 that gets the NMEA data from the EM-406,
//stores it to the GPS_short array.  The "time" variable indicates to set the time
//with the RMC sentence upon power up by sending a 1.  Anything else will be disregarded.
//The sentences that get logged are derived from the log_what variable.
void get_GPS(char time)
{
    int x, z;
    char temp, temp2;

	port = 0;
    temp2 = MIN;

    stat0_on();

    for (x = 0; x < 512; x++)
    {
        GPS_short[x] = 0;
    }
    
    if (set_WAAS == 1)
    {

        for (x = 0; x < 5000000; x++);
        qprintf("$PSRF151,01*0F",0); //set for WAAS
        put_char(13);
        put_char(10);
        set_WAAS = 0;
        
    }


    for(z = 0; z < holdoff; z++)
    {
        x = 0;

        U1FCR = 2;    //Reset the Uart FIFO.  There's some stuff at the beginning of the
					//EM-406 stream that can trip us up.

        while(1)//Get the RMC sentence and check for a lock
        {
            temp = get_char();
            
            if (temp == '$')
            {
                
                GPS_short[x] = temp;
                GPS_short[x + 1] = get_char();
                GPS_short[x + 2] = get_char();
                GPS_short[x + 3] = get_char();
    
                if ((GPS_short[x + 3] == 'R') && (GPS_short[x + 1] == 'G'))
                {
                    
                    GPS_short[x + 4] = get_char();
                    GPS_short[x + 5] = get_char();
                    GPS_short[x + 6] = get_char();
                    GPS_short[x + 7] = get_char();
    
                    if (GPS_short[x + 7] != ',')
                    {
                        
                        x = 8;
                        temp = 0;
    
                        while(1)
                        {
                            GPS_short[x] = get_char();
                            
                            if (x == 18)
                            { 
                                if (GPS_short[x] == 'V') 
                                {
                                    x = 0;
                                    break;
                                }
                            }
							
							x++;
    
                            if (GPS_short[x-1] == 10) break;

                        }
    
                        if (GPS_short[x-1] == 10) break;
                        
                    }
                }
            }
            
        }
    }



    if (time == 1)
    {
        set_time();
        
    }

    //By now we've got a lock.  If we're not interested in this sentence,
    //clear out the buffer
    if ((log_what & 0x20) != 0x20)
    {
        for (x = 0; x < 512; x++)
        {
            GPS_short[x] = 0;

        }

        x = -1;
    }

    if ((log_what & 0x10) == 0x10)  //Get the GGA sentence =====================================================
    {
        
        while(1)
        {
            temp = get_char();
            
            if (temp == '$')
            {
                
                GPS_short[x + 1] = temp;
                GPS_short[x + 2] = get_char();
                GPS_short[x + 3] = get_char();
                GPS_short[x + 4] = get_char();
                GPS_short[x + 5] = get_char();
                GPS_short[x + 6] = get_char();
    
                if ((GPS_short[x + 5] == 'G') && (GPS_short[x + 6] == 'A'))
                {
                    x += 7;
                    
                    while(1)
                    {
                        GPS_short[x] = get_char();
						
						x++;

                        if (GPS_short[x-1] == 10) break;
                        
                        
                    }
                    
                    if (GPS_short[x-1] == 10) break;
                }
            }
            
        }
        
    }


    if ((log_what & 0x08) == 0x08)  //Get the GSA sentence =====================================================
    {
        while(1)
        {
            temp = get_char();
            
            if (temp == '$')
            {
                GPS_short[x + 1] = temp;
                GPS_short[x + 2] = get_char();
                GPS_short[x + 3] = get_char();
                GPS_short[x + 4] = get_char();
                GPS_short[x + 5] = get_char();
                GPS_short[x + 6] = get_char();
    
                if ((GPS_short[x + 5] == 'S') && (GPS_short[x + 6] == 'A'))
                {
                    x += 7;

                    while(1)
                    {
                        GPS_short[x] = get_char();
						
						x++;

                        if (GPS_short[x-1] == 10) break;
                        
                        
                    }
                    
                    if (GPS_short[x-1] == 10) break;
                }
            }
            
        }
    }



    if ((log_what & 0x04) == 0x04)  //Get the GSV sentences =====================================================
    {
        while(1)//get the first...
        {
            temp = get_char();
            
            if (temp == '$')
            {
                GPS_short[x + 1] = temp;
                GPS_short[x + 2] = get_char();
                GPS_short[x + 3] = get_char();
                GPS_short[x + 4] = get_char();
                GPS_short[x + 5] = get_char();
                GPS_short[x + 6] = get_char();
    
                if ((GPS_short[x + 5] == 'S') && (GPS_short[x + 6] == 'V'))
                {
                    x += 7;

                    while(1)
                    {
                        GPS_short[x] = get_char();

                        if (GPS_short[x] == 10) break;
                        
                        x++;
                    }
                    
                    while(1)//get the rest...
                    {
                        temp = get_char();
                        
                        if (temp == '$')
                        {
                            GPS_short[x + 1] = temp;
                            GPS_short[x + 2] = get_char();
                            GPS_short[x + 3] = get_char();
                            GPS_short[x + 4] = get_char();
                            GPS_short[x + 5] = get_char();
                            GPS_short[x + 6] = get_char();
                
                            if ((GPS_short[x + 5] == 'S') && (GPS_short[x + 6] == 'V'))
                            {
                                x += 7;
            
                                while(1)
                                {
                                    GPS_short[x] = get_char();
									
                                    x++;
            
                                    if (GPS_short[x-1] == 10) break;
                                    
                                }
                            }
                            
                            else break;
                        }
                    }

                    if (GPS_short[x-1] == 10) break;
                }
            }
            
        }
    }

    
    if ((log_what & 0x02) == 0x02)  //Get the VTG sentence =====================================================
    {
        while(1)
        {
            temp = get_char();
            
            if (temp == '$')
            {
                GPS_short[x + 1] = temp;
                GPS_short[x + 2] = get_char();
                GPS_short[x + 3] = get_char();
                GPS_short[x + 4] = get_char();
                GPS_short[x + 5] = get_char();
                GPS_short[x + 6] = get_char();
    
                if ((GPS_short[x + 5] == 'T') && (GPS_short[x + 6] == 'G'))
                {
                    x += 7;

                    while(1)
                    {
                        GPS_short[x] = get_char();
                        
                        x++;

                        if (GPS_short[x-1] == 10) break;
                    }
                    
                    if (GPS_short[x-1] == 10) break;
                }
            }
            
        }
    }


    if ((log_what & 0x01) == 0x01)  //Get the GLL sentence =====================================================
    {
        while(1)
        {
            temp = get_char();
            
            if (temp == '$')
            {
                GPS_short[x + 1] = temp;
                GPS_short[x + 2] = get_char();
                GPS_short[x + 3] = get_char();
                GPS_short[x + 4] = get_char();
                GPS_short[x + 5] = get_char();
                GPS_short[x + 6] = get_char();
    
                if ((GPS_short[x + 5] == 'L') && (GPS_short[x + 6] == 'L'))
                {
                    x += 7;

                    while(1)
                    {
                        GPS_short[x] = get_char();
                        
                        x++;

                        if (GPS_short[x-1] == 10) break;
                    }
                    
                    if (GPS_short[x-1] == 10) break;
                }
            }
            
        }
    }
    stringSize = x;

    stat0_off();
}

//sets time (UTC) from GPS
void set_time(void)
{
    CCR = 0;
    CCR = 16;		//Enable RTC config

    HOUR = (GPS_short[7]-48) *10 + (GPS_short[8] - 48);
    MIN = (GPS_short[9]-48) *10 + (GPS_short[10] - 48);
    SEC = (GPS_short[11]-48) *10 + (GPS_short[12] - 48);
    
    CCR = 17;		//Run RTC
}



void mode_1(void)
{
    char temp, temp2, temp3;
    int j;

    while(1)
    {
		
		
        if ((sec_delay + min_delay + hour_delay) != 0)
        {
			
            GPS_off();
    
            temp = SEC + sec_delay; 
            temp2 = MIN + min_delay;
            temp3 = HOUR + hour_delay;
    
            if (temp >= 60)
            {
                temp -= 60;
                temp2 += 1;
            }
            ALSEC = temp;//set seconds for wake up
    
            if (temp2 >= 60)
            {
                temp2 -= 60;
                temp3 += 1;
            }
            ALMIN = temp2;//set minutes for wake up
            
            if (temp3 >= 24) temp3 -= 24;
            ALHOUR = temp3;//set hours for wake up
    
            AMR = 248;  //set alarm to read hours, minutes and seconds
            
            
            
            ILR = 3;	//Reset anything in here before enabling the int
			CIIR = 0x00;	//disable counter ints
            PCONP = 0x00000200;   //shut down everything but the RTC
            VICIntEnable |= 0x00002000;//RTC
            ILR = 3;
			
			EXTWAKE = 0x88;//RTC and EINT3
			
			EXTINT = 0x08;//
        
            PCON = 0x02;    //power down
        
            PCONP = 0x001817BE;   //re-enable the chip peripherals
            
            Initialize();   //start back up
        
            setup_uart0(4800,0);
			setup_uart1(115200,0);
        
            VICIntEnClr = 0x00002000;
			
			if (IOPIN0 & 0x00100000) return;//P0.20 has BT_con
        
			GPS_on();

            if (WAAS == 1) set_WAAS = 1;  //command to issue WAAS enable
        }
		

        //Stop or Standby
        if ((IOPIN0 & 0x00010000) == 0)
        {
            while((IOPIN0 & 0x00010000) == 0)
            {
                stat0_on();
                for (j = 0; j < 500000; j++);
                stat0_off();
                for (j = 0; j < 500000; j++);
                
            }
        }
        
        
        if(mode == 2)//check for external power
        {
			stat0_off();
			stat1_off();
			stat2_off();
			
			GPS_off();
			VICIntEnClr = 0x00002000;//make sure the RTC int is disabled
			PCONP = 0;//shut down everything
			
			EXTWAKE = 0x0A;//EINT1
			EXTINT = 0x0A;//clear the external ints
			PCON = 0x02;    //power down
			//Sleepy, sleepy, sleep...
			PCONP = 0x001817BE;   //re-enable the chip peripherals
			
			Initialize();   //start back up
            
			setup_uart0(4800,0);
			
			if (IOPIN0 & 0x00000008) return;//P0.3 has BT_con
		
			GPS_on();

			sd_raw_init();

			if (WAAS == 1) set_WAAS = 1;  //command to issue WAAS enable
			
        }

        get_GPS(0);
		
        store_GPS();

    }


}



//Stores GPS data to the SD card
void store_GPS(void)
{
    int j;
    signed int k;
    
    stat2_on();

    if ((sec_delay + min_delay + hour_delay) != 0)
    {
        PINSEL0 = 0x00051505;	// enable uart0, enable uart1, enable SPI0 but with SSEL as IO

        for (j = 0; j < 300000; j++);

        if (sd_raw_init() ==  0)    //have to re-init the SD card after power cycle
        {
            stat2_off();
            
			GPS_off();
            
            stat1_on();
            while(1)
            {
                stat0_on();
                for (j = 0; j < 800000; j++);
                stat0_off();
                for (j = 0; j < 800000; j++);
               
            }
        }
    }


	k = fat16_write_file(fd, (unsigned char*)GPS_short, stringSize);
	

    if (k < 0)
    {

        while(1)
        {
            stat0_on();
            for (j = 0; j < 800000; j++);
            stat0_off();
            for (j = 0; j < 800000; j++);
           
        }
    }

	sd_raw_sync();


    if ((sec_delay + min_delay + hour_delay) != 0)
    {
        PINSEL0 = 0x00050005;   // enable uart0, enable uart1, disable SPI0   
    
        IODIR0 &= 0xFFFFFF0F;   //make sure SD lines are hi Z
    }

    stat2_off();
    return;

}


//Returns ASCII Decimal and Hex values
short bin2Hex(short x)
{
	short temp = x & 0x000F;
	temp += 48;
	if (temp > 58) temp += 7;

	return temp;
}



//Prints a string including variables
void qprintf(const char *nate, int my_byte)
{
  	int	  temp;
    short i, k, m;
    //short high_byte = 0, low_byte = 0;
    //short temp2;
    
    short decimal_output[9];
    
    for(i = 0 ; ; i++)
    {
        //delay_ms(3);
        
        k = nate[i];

        if (k == '\0') break;
        
        else if (k == '%') //Print var
        {
            i++;
            k = nate[i];

            if (k == '\0') break;
        	
            else if (k == '\\') //Print special characters
            {
                i++;
                k = nate[i];
                
                put_char(k);
                

            } //End Special Characters
            else if (k == 'b') //Print Binary
            {
                for( m = 0 ; m < 32 ; m++ )
                {
                    if ((my_byte & 0x80000000) == 0x80000000) put_char('1');
                    if ((my_byte & 0x80000000) == 0) put_char('0');
                    if (((m+1) % 4) == 0) put_char(' ');
                    
                    my_byte = my_byte << 1;
                }
            } //End Binary               
            else if (k == 'd') //Print Decimal
            {
                //Print negative sign and take 2's compliment
                
                if(my_byte < 0)
                {
                    put_char('-');
                    my_byte *= -1;
                }
                
                
                if (my_byte == 0)
                    put_char('0');
                else
                {
                    //Divide number by a series of 10s
                    for(m = 9 ; my_byte > 0 ; m--)
                    {
                        temp = my_byte % 10;
                        decimal_output[m] = temp;
                        my_byte = my_byte / 10;               
                    }
                
                    for(m++ ; m < 10 ; m++)
                        put_char(bin2Hex(decimal_output[m]));
                }
    
            } //End Decimal
			
			/*
            else if (k == 'h') //Print Hex
            {
                //New trick 3-15-04
                put_char('0');
                put_char('x');
                
                for (m = 0; m < 8; m++)
                {
                	temp = my_byte & 0xF0000000;
                	temp2 = temp >>= 28;
                	put_char(bin2Hex(temp2));
                	my_byte <<= 4;
                }
                
                
                
                //if(my_byte > 0x00FF)
                //{
                    //put_char(bin2Hex(my_byte.high8 >> 4));
                    //put_char(bin2Hex(my_byte.high8 & 0b.0000.1111));
                //}

                //put_char(bin2Hex(my_byte.low8 >> 4));
                //put_char(bin2Hex(my_byte.low8 & 0b.0000.1111));

                //high_byte.3 = my_byte.7;
                //high_byte.2 = my_byte.6;
                //high_byte.1 = my_byte.5;
                //high_byte.0 = my_byte.4;
            
                //low_byte.3 = my_byte.3;
                //low_byte.2 = my_byte.2;
                //low_byte.1 = my_byte.1;
                //low_byte.0 = my_byte.0;
        
                //put_char('0');
                //put_char('x');
            
                //put_char(bin2Hex(high_byte));
                //put_char(bin2Hex(low_byte));
            } //End Hex
			*/
			
            else if (k == 'f') //Print Float
            {
                put_char('!');
            } //End Float
			
            else if (k == 'u') //Print Direct Character
            {
                //All ascii characters below 20 are special and screwy characters
                //if(my_byte > 20) 
                    put_char(my_byte);
            } //End Direct
                        
        } //End Special Chars           

        else
            put_char(k);
    }    
}


void change_settings(void)
{
	char a, b;
	int j;
	
	while (1)
	{
		qprintf("\r\n",0);
		
		qprintf("\r\nGeochron Settings Menu\r\n",0);
		qprintf("========================================\r\n",0);
		
		qprintf("1) Change mode, currently %d\r\n",mode);
		qprintf("2) Change Log What, currently ",0);
		if (log_what & 0x20) qprintf("RMC",0);//
		if (log_what & 0x10) qprintf(";GGA",0);//
		if (log_what & 0x08) qprintf(";GSA",0);//
		if (log_what & 0x04) qprintf(";GSV",0);//
		if (log_what & 0x02) qprintf(";VTG",0);
		if (log_what & 0x01) qprintf(";GLL",0);
		qprintf("\r\n",0);
		
		qprintf("3) Change Time Between Logs, currently ",0);
		put_char(hour_delay / 10 + 48);
		put_char(hour_delay % 10 + 48);
		put_char(':');
		put_char(min_delay / 10 + 48);
		put_char(min_delay % 10 + 48);
		put_char(':');
		put_char(sec_delay / 10 + 48);
		put_char(sec_delay % 10 + 48);
		qprintf("\r\n",0);
		
		qprintf("4) Change Holdoff, currently %d\r\n",holdoff);
		qprintf("5) Change WAAS, currently %d\r\n",WAAS);
		qprintf("6) Change Bluetooth Active, currently %d\r\n",BT_active);
		
		qprintf("9) Return to main menu\r\n",0);
		

		U1FCR = 0x02;   //Clear the RX fifo

		if (port == 1) a = getc2();
              
		if (a == '1')
		{
			mode++;
			if (mode >= 3) mode = 0;
		}
		
		else if (a == '2')
		{
			for (j = 0; j < 500; j++) RX_array1[j] = 0;
			while(1)
			{
				qprintf("\r\n",0);
				qprintf("\r\nPress the number of the sentence you wish to activate/de-activate\r\n",0);
				qprintf("1) RMC ",0);
				if (log_what & 0x20) qprintf("on\r\n",0);//
				else qprintf("off\r\n",0);
				qprintf("2) GGA ",0);
				if (log_what & 0x10) qprintf("on\r\n",0);//
				else qprintf("off\r\n",0);
				qprintf("3) GSA ",0);
				if (log_what & 0x08) qprintf("on\r\n",0);//
				else qprintf("off\r\n",0);
				qprintf("4) GSV ",0);
				if (log_what & 0x04) qprintf("on\r\n",0);//
				else qprintf("off\r\n",0);
				qprintf("9) Quit\r\n ",0);
				
				b = getc2();
				
				if (b == '1') log_what ^= 0x20;
				else if (b == '2') log_what ^= 0x10;
				else if (b == '3') log_what ^= 0x08;
				else if (b == '4') log_what ^= 0x04;
				
				else if (b == '9') break;
			}
				
		}
		
		else if (a == '3')
		{
			qprintf("\r\n",0);
			qprintf("Enter your new time in the format \r\n",0);
			put_char(34);
			qprintf("HH:MM:SS",0);
			put_char(34);
			qprintf(" (we'll put the colons in for you)\r\n>",0);
			
			b = getc2();
			hour_delay = (b-48)*10;
			put_char(b);
			b = getc2();
			hour_delay += b-48;
			put_char(b);
			
			put_char(':');
			
			b = getc2();
			min_delay = (b-48)*10;
			put_char(b);
			b = getc2();
			min_delay += b-48;
			put_char(b);
			
			put_char(':');
			
			b = getc2();
			sec_delay = (b-48)*10;
			put_char(b);
			b = getc2();
			sec_delay += b-48;
			put_char(b);
			
			if (hour_delay >= 24) hour_delay = 0;
			if (min_delay >= 60) min_delay = 0;
			if (sec_delay >= 60) sec_delay = 10;
		}
		
		else if (a == '4')
		{
			qprintf("\r\n",0);
			qprintf("Enter your new Holdoff, 0 to 99, followed by <enter>\r\n>",0);
			
			j = 0;
			
			
			while(1)
			{
				b = getc2();
				put_char(b);
				if ((b == 10) | (b == 13)) break;
				
				if (j != 0) holdoff *= 10;
				holdoff += b-48;
				
				j++;
				
			}

			if (holdoff >= 99) holdoff = 5;
		
		}
		
		else if (a == '5')
		{
			qprintf("\r\n",0);
			WAAS ^= 1;
			
		}
		
		else if (a == '6')
		{
			qprintf("\r\n",0);
			BT_active ^= 1;
			
		}
		
		else if (a == '9')
		{
			qprintf("\r\n\r\n",0);
			qprintf("Saving new config file, returning to main menu...\r\n\r\n",0);
			Log_init();
			return;
		}
	}
		
			
}


